import type { Map, MapEffect, MapObject, Tile, UUID } from '@/application/types'
import { defineStore } from 'pinia'

export type TeleportSettings = {
  toMap: Map | null
  toPositionX: number
  toPositionY: number
  toRotation: number
}

export const useMapEditorStore = defineStore('mapEditor', {
  state: () => {
    return {
      active: true,
      mapId: '',
      tool: 'move',
      drawMode: 'tile',
      eraserMode: 'tile',
      tileList: [] as Tile[],
      mapObjectList: [] as MapObject[],
      selectedTile: '',
      selectedMapObject: null as MapObject | null,
      isTileListModalShown: false,
      isMapObjectListModalShown: false,
      isMapListModalShown: false,
      isCreateMapModalShown: false,
      isSettingsModalShown: false,
      shouldClearTiles: false,
      mapSettings: {
        name: '',
        width: 0,
        height: 0,
        pvp: false,
        mapEffects: [] as MapEffect[]
      },
      teleportSettings: {
        toMap: null,
        toPositionX: 0,
        toPositionY: 0,
        toRotation: 0
      } as TeleportSettings
    }
  },
  actions: {
    toggleActive() {
      if (this.active) this.reset()
      this.active = !this.active
    },
    setMapId(mapId: UUID) {
      this.mapId = mapId
    },
    setMapName(name: string) {
      this.mapSettings.name = name
    },
    setMapWidth(width: number) {
      this.mapSettings.width = width
    },
    setMapHeight(height: number) {
      this.mapSettings.height = height
    },
    setMapPvp(pvp: boolean) {
      this.mapSettings.pvp = pvp
    },
    setMapEffects(mapEffects: MapEffect[]) {
      this.mapSettings.mapEffects = mapEffects
    },
    setTool(tool: string) {
      this.tool = tool
    },
    setDrawMode(mode: string) {
      this.drawMode = mode
    },
    setEraserMode(mode: string) {
      this.eraserMode = mode
    },
    setMapList(maps: Map[]) {
      this.mapList = maps
    },
    setTileList(tiles: Tile[]) {
      this.tileList = tiles
    },
    setMapObjectList(objects: MapObject[]) {
      this.mapObjectList = objects
    },
    setSelectedTile(tile: string) {
      this.selectedTile = tile
    },
    setSelectedMapObject(object: MapObject) {
      this.selectedMapObject = object
    },
    toggleSettingsModal() {
      this.isSettingsModalShown = !this.isSettingsModalShown
    },
    toggleMapListModal() {
      this.isMapListModalShown = !this.isMapListModalShown
      this.isCreateMapModalShown = false
    },
    toggleCreateMapModal() {
      this.isCreateMapModalShown = !this.isCreateMapModalShown
    },
    setTeleportSettings(teleportSettings: TeleportSettings) {
      this.teleportSettings = teleportSettings
    },
    triggerClearTiles() {
      this.shouldClearTiles = true
    },
    resetClearTilesFlag() {
      this.shouldClearTiles = false
    },
    reset() {
      this.mapId = ''
      this.mapList = []
      this.tileList = []
      this.mapObjectList = []
      this.tool = 'move'
      this.drawMode = 'tile'
      this.selectedTile = ''
      this.selectedMapObject = null
      this.isTileListModalShown = false
      this.isMapObjectListModalShown = false
      this.isSettingsModalShown = false
      this.isMapListModalShown = false
      this.isCreateMapModalShown = false
      this.shouldClearTiles = false
    }
  }
})
